//  IoT本格学習キット スマートリモコン
//  7-2. Scratch3用プログラム（インターネット接続なし）

// ①ライブラリを読み込み
#include <WiFi.h>
#include <DHT.h>
#include <FS.h>
#include <SPIFFS.h>
#include <ESPAsyncWebServer.h>

// ####################### ②Wi-Fi設定(環境設定) #######################
const char *ssid      = "##### SSID #####";
const char *password  = "### PASSWORD ###";
IPAddress ip(192, 168, 1, 123);  // IPアドレス（本機が利用するIP）
IPAddress gateway(192, 168, 1, 1);  // デフォルトゲートウェイ
IPAddress subnet(255, 255, 255, 0); // サブネットマスク
// ###################################################################

// ③ピン配置
const byte LED_PIN = 22;   // LED緑
const byte IR_R_PIN = 36;  // リモコン受信
const byte IR_S_PIN = 23;  // リモコン送信
const byte LIGH_SNR = 39;  // 照度センサ(A18)
const byte TEMP_SNR = 21;  // 温・湿度センサ

// ④グローバル変数定義
AsyncWebServer server(80);        // Port80番
unsigned short irData[1500];
bool ledFlag = true;      // LED制御フラグ

void setup(void) {
  // ⑤Serial設定
  Serial.begin(115200);
  Serial.println ( );
  // ⑤SPIFFS開始
  SPIFFS.begin();
  // ⑥LED設定
  pinMode ( LED_PIN, OUTPUT );
  pinMode ( IR_R_PIN, INPUT );
  pinMode ( IR_S_PIN, OUTPUT );
  pinMode ( LIGH_SNR, INPUT );
  // ⑦無線Wi-Fi接続
  WiFi.config( ip, gateway, subnet );
  WiFi.begin ( ssid, password );

  // ⑧Wi-Fi接続処理（接続するまで無限ループ）
  while ( WiFi.status() != WL_CONNECTED ) {
    // ⑨LEDを１秒毎に点滅する
    ledFlag = !ledFlag;
    digitalWrite(LED_PIN, ledFlag);
    // ⑩１秒間Wait
    delay ( 1000 );
    Serial.print ( "." );
  }
  // ⑪Wi-Fi接続できたのでシリアルモニターにIPアドレス表示
  Serial.print ( "Wi-Fi Connected! IP address: " );
  Serial.println ( WiFi.localIP() );
  // ⑫LED点灯（Wi-Fi接続状態）
  digitalWrite ( LED_PIN, true );

  // ⑬WebServerアクセス処理[LED制御]
  server.on("/led", HTTP_GET, [](AsyncWebServerRequest *request){
      Serial.println ( "webServer led" );
      String setLedVal = request->getParam("o")->value();
      if (setLedVal == "ON") {
        digitalWrite ( LED_PIN, true );
      } else {
        digitalWrite ( LED_PIN, false );
      }
      // 応答処理
      request->send(200, "text/plain", "OK");
  });
  // ⑭WebServerアクセス処理[センサー情報更新]
  server.on("/update", HTTP_GET, [](AsyncWebServerRequest *request){
      Serial.println ( "webServer update" );
      // リモコン送信
      String sendMsg = updateSensor();
      // 応答処理
      request->send(200, "application/json", sendMsg);
  });
  // ⑮WebServerアクセス処理[リモコン送信]
  server.on("/send", HTTP_GET, [](AsyncWebServerRequest *request){
      Serial.println ( "webServer send:" + request->getParam("n")->value() );
      // リモコン送信
      String replyResult = "NG";
      if ( contRemocon(request->getParam("n")->value()) ){
        replyResult = "OK";
      }
      // 応答処理
      request->send(200, "text/plain", replyResult);
  });
  // ⑯WebServerアクセス処理[リモコン受信]
  server.on("/recv", HTTP_GET, [](AsyncWebServerRequest *request){
      Serial.println ( "webServer recv:" + request->getParam("n")->value() );
      // リモコン受信設定
      String replyResult = "NG";
      if ( setRemocon(request->getParam("n")->value()) ){
        replyResult = "OK";
      }
      // 応答処理
      Serial.println ( "send recv response." );
      request->send(200, "text/plain", replyResult);
  });
  // ⑰WebServerヘッダ設定及び起動
  DefaultHeaders::Instance().addHeader("Access-Control-Allow-Origin", "*");
  server.begin();
  Serial.println ( "Web server started" );
}

void loop(void){

}

// ⑱センサー情報を更新し送信データ作成
String updateSensor(){
  DHT dht(TEMP_SNR, DHT11);
  float g_temperature = dht.readTemperature();
  float g_humidity = dht.readHumidity();
  unsigned short g_light = analogRead(LIGH_SNR) * 100 / 4095;
  String messageS = "{\"t\":" + String(int(g_temperature)) + ",\"h\":" + String(int(g_humidity)) + ",\"l\":" + String(g_light) + "}";
  return messageS;
}
